"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PmdTool = void 0;
const BuildOutput_1 = require("./BuildOutput");
const BaseTool_1 = require("./BaseTool");
const path = require("path");
const fs = require("fs");
const xml2js = require("xml2js");
const tl = require("azure-pipelines-task-lib/task");
/**
 * An object that is able to configure the build to run PMD and identify and parse PMD reports
 *
 * @export
 * @class PmdReportParser
 * @implements {IAnalysisToolReportParser}
 */
class PmdTool extends BaseTool_1.BaseTool {
    constructor(buildOutput, boolInputName) {
        super('PMD', buildOutput, boolInputName);
    }
    /**
     * Configures the provided ToolRunner instance with arguments which will invoke the tool represented by this class.
     * @param toolRunner
     * @returns {ToolRunner} ToolRunner instance with arguments applied
     */
    configureBuild(toolRunner) {
        if (this.isEnabled()) {
            console.log(tl.loc('codeAnalysis_ToolIsEnabled'), this.toolName);
            switch (this.buildOutput.buildEngine) {
                case BuildOutput_1.BuildEngine.Maven:
                    toolRunner.arg(['pmd:pmd']);
                    break;
                case BuildOutput_1.BuildEngine.Gradle:
                    let initScriptPath = path.join(__dirname, '..', 'pmd.gradle');
                    toolRunner.arg(['-I', initScriptPath]);
                    break;
                default:
                    break;
            }
        }
        return toolRunner;
    }
    /**
     * Implementers must specify where the XML reports are located
     */
    getBuildReportDir(output) {
        switch (this.buildOutput.buildEngine) {
            case BuildOutput_1.BuildEngine.Maven:
                return path.join(output.moduleRoot);
            case BuildOutput_1.BuildEngine.Gradle:
                return path.join(output.moduleRoot, 'reports', 'pmd');
            default:
                throw new Error('No such build engine ' + this.buildOutput.buildEngine);
        }
    }
    /**
     * Report parser that extracts the number of affected files and the number of violations from a report
     *
     * @returns a tuple of [affected_file_count, violation_count]
     */
    parseXmlReport(xmlReport, moduleName) {
        let fileCount = 0;
        let violationCount = 0;
        let reportContent = fs.readFileSync(xmlReport, 'utf-8');
        xml2js.parseString(reportContent, (err, data) => {
            // If the file is not XML, or is not from PMD, return immediately
            if (!data || !data.pmd) {
                tl.debug(`[CA] Empty or unrecognized PMD xml report ${xmlReport}`);
                return null;
            }
            if (!data.pmd.file || data.pmd.file.length === 0) { // No files with violations, return now that it has been marked for upload
                tl.debug(`[CA] A PMD report was found for module '${moduleName}' but it contains no violations`);
                return null;
            }
            data.pmd.file.forEach((file) => {
                if (file.violation) {
                    fileCount++;
                    violationCount += file.violation.length;
                }
            });
            tl.debug(`[CA] A PMD report was found for for module '${moduleName}' containing ${violationCount} issues - ${xmlReport}`);
        });
        return [violationCount, fileCount];
    }
}
exports.PmdTool = PmdTool;
